@@ -15,7 +15,7 @@ namedList <- function(...) {
 ##' @return those elements of \code{nms} which are "checking options"
 ##' @author Martin Maechler
 .get.checkingOpts <- function(nms)
-    nms[grepl("^check\\.(?!conv|rankX)", nms, perl=TRUE)]
+    nms[grepl("^check\\.(?!conv|rankX|scaleX)", nms, perl=TRUE)]
 
 
 ##' Check check.conv.*() options and produce good error message
@@ -72,16 +72,16 @@ lmerControl <-
 	     use.last.params=FALSE,
 	     sparseX=FALSE,
 	     ## input checking options:
-	     check.nobs.vs.rankZ="warningSmall",
+	     check.nobs.vs.rankZ="ignore", ## "warningSmall",
 	     check.nobs.vs.nlev="stop",
 	     check.nlev.gtreq.5="ignore",
 	     check.nlev.gtr.1="stop",
 	     check.nobs.vs.nRE="stop",
 	     check.rankX = c("message+drop.cols",
-	     "silent.drop.cols", "warn+drop.cols",
-	     "stop.deficient", "ignore"),
-	     check.scaleX = "warning",
-	     ##TODO? c("warning","message+scale", "silent.scale","ignore"),
+                             "silent.drop.cols", "warn+drop.cols",
+                 	     "stop.deficient", "ignore"),
+	     check.scaleX = c("warning","stop","silent.rescale",
+                              "message+rescale","warn+rescale","ignore"),
 	     check.formula.LHS = "stop",
 	     ## convergence options
 	     check.conv.grad	 = .makeCC("warning", tol = 2e-3, relTol = NULL),
@@ -97,12 +97,19 @@ lmerControl <-
     stopifnot(is.list(optCtrl))
 
     if (!is.null(lmerOpts <- getOption("lmerControl"))) {
-        for (arg in .get.checkingOpts(names(lmerOpts))) {
+        nn <- names(lmerOpts)
+        nn.ok <- .get.checkingOpts(names(lmerOpts))
+        if (length(nn.ignored <- setdiff(nn,nn.ok))>0) {
+            warning("some options in ",shQuote("getOption('lmerControl')"),
+                    " ignored : ",paste(nn.ignored,collapse=", "))
+        }
+        for (arg in nn.ok) {
             if (do.call(missing,list(arg))) ## only if missing from explicit arguments
                 assign(arg,lmerOpts[[arg]])
         }
     }
     check.rankX <- match.arg(check.rankX)# ==> can abbreviate
+    check.scaleX <- match.arg(check.scaleX)# ==> can abbreviate
 
     ## compatibility and convenience, caller can specify action string only:
     me <- sys.function()
@@ -142,7 +149,7 @@ glmerControl <-
 	     tolPwrss = 1e-7,
 	     compDev = TRUE,
 	     ## input checking options
-	     check.nobs.vs.rankZ="warningSmall",
+	     check.nobs.vs.rankZ="ignore", ## "warningSmall",
 	     check.nobs.vs.nlev="stop",
 	     check.nlev.gtreq.5="ignore",
 	     check.nlev.gtr.1="stop",
@@ -171,7 +178,13 @@ glmerControl <-
 	optimizer <- replicate(2,optimizer) # works evevn when optimizer is function
     }
     if (!is.null(glmerOpts <- getOption("glmerControl"))) {
-        for (arg in .get.checkingOpts(names(glmerOpts))) {
+        nn <- names(glmerOpts)
+        nn.ok <- .get.checkingOpts(names(glmerOpts))
+        if (length(nn.ignored <- setdiff(nn,nn.ok))>0) {
+            warning("some options in ",shQuote("getOption('glmerControl')"),
+                    " ignored : ",paste(nn.ignored,collapse=", "))
+        }
+        for (arg in nn.ok) {
             if (do.call(missing,list(arg))) ## only if missing from explicit arguments
                 assign(arg, glmerOpts[[arg]])
         }
