@@ -23,21 +23,24 @@ setNBdisp <- function(object,theta) {
   object
 }
 
-refitNB <- function(object,theta) {
-  refit(setNBdisp(object,theta))
+refitNB <- function(object, theta, control = NULL) {
+  refit(setNBdisp(object, theta), control = control)
 }
 
 optTheta <- function(object,
-                     interval=c(-5,5),
-                     maxit=20,
-                     verbose=FALSE) {
+                     interval = c(-5,5),
+                     maxit = 20,
+                     verbose = FALSE,
+                     control = NULL) {
   lastfit <- object
   evalcnt <- 0
   optval <- optimize(function(t) {
       ## FIXME: kluge to retain last value and evaluation count
       ## Perhaps use a reference class object to keep track of this
       ## auxilliary information?  DB
-      dev <- deviance(lastfit <<- refitNB(lastfit,theta=exp(t)))
+      dev <- deviance(lastfit <<- refitNB(lastfit,
+                                          theta = exp(t),
+                                          control = control))
       evalcnt <<- evalcnt+1
       if (verbose) cat(evalcnt,exp(t),dev,"\n")
       dev
@@ -62,18 +65,21 @@ est_theta <- function(object) {
 ## I (MM) don't know how to use roxygen for that..
 
 ##' glmer() for Negative Binomial
-##' @param ... formula, data, etc: the arguments for \code{\link{glmer}(..)} (apart from \code{family}!).
+##' @param ... formula, data, etc: the arguments for
+##' \code{\link{glmer}(..)} (apart from \code{family}!).
 ##' @param interval interval in which to start the optimization
-##' @param verbose logical indicating how much progress information should be printed.
+##' @param verbose logical indicating how much progress information
+##' should be printed.
 ##' @export
 glmer.nb <- function(..., interval = log(th)+c(-3,3), verbose=FALSE)
 {
-  g0 <- glmer(..., family=poisson)
-  th <- est_theta(g0)
-  if(verbose) cat("th := est_theta(glmer(..)) =", format(th),"\n")
-  g1 <- update(g0, family = negative.binomial(theta=th))
-  ## if (is.null(interval)) interval <- log(th)+c(-3,3)
-  optTheta(g1, interval=interval, verbose=verbose)
+    control <- list(...)$control
+    g0 <- glmer(..., family = poisson)
+    th <- est_theta(g0)
+    if(verbose) cat("th := est_theta(glmer(..)) =", format(th),"\n")
+    g1 <- update(g0, family = negative.binomial(theta=th))
+    ## if (is.null(interval)) interval <- log(th)+c(-3,3)
+    optTheta(g1, interval = interval, verbose = verbose, control = control)
 }
 
 ## do we want to facilitate profiling on theta??
