@@ -5,6 +5,10 @@ noReForm <- function(re.form) {
         (is(re.form,"formula") && length(re.form)==2 && identical(re.form[[2]],0))
 }
 
+reOnly <- function(f) {
+    reformulate(paste0("(",sapply(findbars(f),deparse),")"))
+}
+
 reFormHack <- function(re.form,ReForm,REForm,REform) {
     if (!missing(ReForm)) {
         message(shQuote("re.form")," is now preferred to ",shQuote("ReForm"))
@@ -145,8 +149,14 @@ mkNewReTrms <- function(object, newdata, re.form=NULL, ReForm,
                  " for grouping variables unless allow.new.levels is TRUE")
         unames <- unique(sort(names(ReTrms$cnms)))  ## FIXME: same as names(ReTrms$flist) ?
         ## convert numeric grouping variables to factors as necessary
-        for (i in all.vars(RHSForm(re.form))) {
-            rfd[[i]] <- factor(rfd[[i]])
+        ## must use all.vars() for examples
+        ## for (i in all.vars(RHSForm(re.form))) {
+        ## TO DO: should restrict attention to grouping factors only
+        getgrpvars <- function(x) all.vars(x[[3]])
+        all.grp.vars <- unique(unlist(lapply(findbars(re.form),getgrpvars)))
+        for (i in all.grp.vars) {
+            if (!is.matrix(rfd[[i]]))
+                rfd[[i]] <- factor(rfd[[i]])
         }
         Rfacs <- lapply(setNames(unames,unames),
                         function(x) factor(eval(parse(text=x),envir=rfd)))
@@ -300,8 +310,11 @@ predict.merMod <- function(object, newdata=NULL, newparams=NULL, newX=NULL,
             RHS <- formula(substitute(~R,
                               list(R=RHSForm(formula(object,fixed.only=TRUE)))))
             Terms <- terms(object,fixed.only=TRUE)
-            isFac <- vapply(mf <- model.frame(object,fixed.only=TRUE),
+            isFac <- vapply(mf <- model.frame(object,
+                                              fixed.only=TRUE),
                             is,"factor",FUN.VALUE=TRUE)
+            ## ignore response variable
+            isFac[attr(Terms,"response")] <- FALSE
             orig_levs <- if (length(isFac)==0) NULL else lapply(mf[isFac],levels)
             X <- model.matrix(RHS, mfnew <- model.frame(delete.response(Terms),
                                                         newdata,
@@ -324,9 +337,8 @@ predict.merMod <- function(object, newdata=NULL, newparams=NULL, newX=NULL,
         pred <- pred+offset
         if (!noReForm(re.form)) {
             if (is.null(re.form)) {
-                ## original formula, minus response
-                re.form <- LHSForm(formula(object))
-                ReTrms <- mkReTrms(findbars(re.form[[2]]), newdata)
+		re.form <- reOnly(formula(object)) # RE formula only
+                ReTrms <- mkReTrms(findbars(re.form), newdata)
             }
             newRE <- mkNewReTrms(object,newdata,re.form,na.action=na.action,
                                  allow.new.levels=allow.new.levels)
@@ -387,7 +399,7 @@ simulate.merMod <- function(object, nsim = 1, seed = NULL, use.u = FALSE,
                             allow.new.levels=FALSE, na.action=na.pass, ...) {
     mc <- match.call()
     mc[[1]] <- quote(lme4::.simulateFun)
-    return(eval(mc, parent.frame(1L)))
+    eval(mc, parent.frame(1L))
 }
 
 .simulateFun <- function(object, nsim = 1, seed = NULL, use.u = FALSE,
@@ -457,9 +469,8 @@ simulate.merMod <- function(object, nsim = 1, seed = NULL, use.u = FALSE,
         }
         re.form <- if (use.u) NULL else ~0
     }
-    if (is.null(re.form)) {
-        ## original formula, minus response
-        re.form <- LHSForm(formula(object))
+    if (is.null(re.form)) { # formula w/o response
+	re.form <- noLHSform(formula(object))
     }
     if(!is.null(seed)) set.seed(seed)
     if(!exists(".Random.seed", envir = .GlobalEnv))
@@ -471,15 +482,17 @@ simulate.merMod <- function(object, nsim = 1, seed = NULL, use.u = FALSE,
     link <- if (isGLMM(object)) "response"
 
     ## predictions, conditioned as specified, on link scale
-    ## do **NOT** use na.action as specified here (inherit
-    ## from object instead, for consistency)
+    ## previously: do **NOT** use na.action as specified here (inherit
+    ##     from object instead, for consistency)
+    ## now: use na.omit, because we have to match up
+    ##    with whatever is done in mkNewReTrms
     etapred <- predict(object, newdata=newdata, re.form=re.form,
-                       type="link")
+                       type="link", na.action=na.omit)
 
     ## now add random components:
     ##  only the ones we did *not* condition on
 
-    ## compre.form <- LHSForm(formula(object))
+    ## compre.form <- noLHSform(formula(object))
     ## construct RE formula ONLY: leave out fixed terms,
     ##   which might have loose terms like offsets in them ...
     fb <- findbars(formula(object))
@@ -542,24 +555,48 @@ simulate.merMod <- function(object, nsim = 1, seed = NULL, use.u = FALSE,
         val <- as.data.frame(val)
     }  else class(val) <- "data.frame"
     names(val) <- paste("sim", seq_len(nsim), sep="_")
-    if (is.null(nm <- names(fitted(object)))) nm <- seq(n)
+    ## have not yet filled in NAs, so need to use names of fitted
+    ## object NOT including values with NAs
+    f <- fitted(object)
+    nm <- names(f)[!is.na(f)]  
+    if (length(nm)==0) nm <- seq(n)
     row.names(val) <- nm
 
-    if (!missing(na.action)) {
-        if (!is.null(fit.na.action <- attr(model.frame(object),"na.action"))) {
-            class.na.action <- class(attr(na.action(NA),"na.action"))
-            if (class.na.action != class(fit.na.action)) {
-                ## hack to override action where explicitly specified
-                class(fit.na.action) <- class.na.action
-                val <- as.data.frame(lapply(val,napredict,
-                      omit=fit.na.action))
-                ## reconstruct names
-                row.names(val) <- names(napredict(fitted(object),
-                                                  omit=fit.na.action))
-            }
+    fit.na.action <- attr(model.frame(object),"na.action")
+
+    if (!missing(na.action) &&  !is.null(fit.na.action)) {
+        ## retrieve name of na.action type ("omit", "exclude", "pass")
+        class.na.action <- class(attr(na.action(NA),"na.action"))
+        if (class.na.action != class(fit.na.action)) {
+            ## hack to override action where explicitly specified
+            class(fit.na.action) <- class.na.action
         }
     }
 
+    if (is.matrix(val[[1]])) {
+        ## have to handle binomial response matrices differently --
+        ## fill in NAs as appropriate in *both* columns
+        val <- lapply(val,function(x) { apply(x,2,napredict,
+                                       omit=fit.na.action) })
+        ## have to put this back into a (weird) data frame again,
+        ## carefully (should do the napredict stuff
+        ## earlier, so we don't have to redo this transformation!)
+        class(val) <- "data.frame"
+    } else {
+        val <- as.data.frame(lapply(val,napredict, omit=fit.na.action))
+    }
+
+    ## reconstruct names: first get rid of NAs, then refill them
+    ## as appropriate based on fit.na.action (which may be different
+    ## from the original model's na.action spec)
+    if (length(nm2 <- names(napredict(na.omit(f),
+                                        omit=fit.na.action)))>0)
+        row.names(val) <- nm2
+
+    ## as.data.frame(lapply(...)) blows away na.action attribute,
+    ##  so we have to re-assign here
+    attr(val,"na.action") <- fit.na.action
+
     attr(val, "seed") <- RNGstate
     val
 }
@@ -627,15 +664,37 @@ poisson_simfun <- function(object, nsim, ftd=fitted(object)) {
     }
 
 
-## FIXME: need a gamma.shape.merMod method in order for this to work
+##' FIXME: need a gamma.shape.merMod method in order for this to work.
+##'        (see initial shot at gamma.shape.merMod below)
 Gamma_simfun <- function(object, nsim, ftd=fitted(object)) {
     wts <- weights(object)
     if (any(wts != 1)) message("using weights as shape parameters")
-    ftd <- fitted(object)
+    ## ftd <- fitted(object)
     shape <- MASS::gamma.shape(object)$alpha * wts
     rgamma(nsim*length(ftd), shape = shape, rate = shape/ftd)
 }
 
+gamma.shape.merMod <- function(object, ...) {
+    if(!(family(object)$family == "Gamma"))
+        stop("Can not fit gamma shape parameter ",
+             "because Gamma family not used")
+
+    y <- getME(object, "y")
+    mu <- getME(object, "mu")
+    w <- weights(object)
+                                        # Sec 8.3.2 (MN)
+    L <- w*(log(y/mu)-((y-mu)/mu))
+    dev <- -2*sum(L)
+                                        # Eqs. between 8.2 & 8.3 (MN)
+    Dbar <- dev/length(y)
+    alpha <- (6+2*Dbar)/(Dbar*(6+Dbar))
+                                        # FIXME: obtain standard error
+    res <- list(alpha = alpha, SE = NA)
+    class(res) <- "gamma.shape"
+    res
+}
+
+
 ## FIXME: include without inducing SuppDists dependency?
 ## inverse.gaussian_simfun <- function(object, nsim, ftd=fitted(object)) {
 ##     if(is.null(tryCatch(loadNamespace("SuppDists"),
