@@ -1,7 +1,6 @@
 ## generalized linear mixed model
 stopifnot(suppressPackageStartupMessages(require(lme4)))
 options(show.signif.stars = FALSE)
-(testLevel <- if (nzchar(s <- Sys.getenv("LME4_TEST_LEVEL"))) as.numeric(s) else 1)
 
 source(system.file("test-tools-1.R", package = "Matrix"), keep.source = FALSE)
 ##
@@ -81,11 +80,11 @@ stopifnot(is((cm2 <- coef(m2)), "coef.mer"),
                       -1.12782184600404, -1.57946627431248),
                     ##c(-1.3766013, -1.0058773,
                     ##  -1.1430128, -1.5922817),
-		    tol = 5.e-4,
+		    tolerance = 5.e-4,
                     check.attributes=FALSE),
-          all.equal(deviance(m2), 100.010030538022, tol=1e-9)
+          all.equal(deviance(m2), 100.010030538022, tolerance=1e-9)
           ## with bobyqa first (AGQ=0), then
-          ##all.equal(deviance(m2), 101.119749563, tol=1e-9)
+          ##all.equal(deviance(m2), 101.119749563, tolerance=1e-9)
           )
 
 ## 32-bit Ubuntu 10.04:
@@ -116,7 +115,7 @@ coef_m1_glmmML <- structure(c(-1.39853234657711, -0.992336901732793, -1.12867036
 
 stopifnot(is((cm1 <- coef(m1b)), "coef.mer"),
 	  dim(cm1$herd) == c(15,4),
-          all.equal(fixef(m1b),fixef(m1),tol=4e-5),
+          all.equal(fixef(m1b),fixef(m1),tolerance=4e-5),
 	  is.all.equal4(fixef(m1b),
                         coef_m1_glmmadmb,
                         coef_m1_lme4.0,
@@ -145,14 +144,14 @@ if (require('MASS', quietly = TRUE)) {
 			structure(-96.13069, nobs = 220L, nall = 220L,
 				  df = 5L, REML = FALSE,
                                   class = "logLik"),
-                        tol = 5e-4, check.attributes = FALSE)
+                        tolerance = 5e-4, check.attributes = FALSE)
 	      ,
 	      all.equal(fixef(fm5),
 			## was		 2.834218798		 -1.367099481
 			c("(Intercept)"= 2.831609490, "trtdiag"= -1.366722631,
 			  ## now	 0.5842291915,		 -1.599148773
 			  "trtencourage"=0.5840147802, "wk2TRUE"=-1.598591346),
-                        tol = 1e-4 )
+                        tolerance = 1e-4 )
 	      )
 }
 
@@ -214,12 +213,12 @@ stopifnot(isTRUE(chkFixed(m0, true.coef = c(1,2))),
           isTRUE(chkFixed(m1, true.coef = c(1,2))))
 (a01 <- anova(m0, m1))
 
-stopifnot(all.equal(a01$Chisq[2], 554.334056, tol=1e-5),
-	  all.equal(a01$logLik, c(-1073.77193, -796.604902), tol=1e-6),
+stopifnot(all.equal(a01$Chisq[2], 554.334056, tolerance=1e-5),
+	  all.equal(a01$logLik, c(-1073.77193, -796.604902), tolerance=1e-6),
           a01$ Df == 3:4,
 	  a01$`Chi Df`[2] == 1)
 
-if (testLevel > 1) {
+if(lme4:::testLevel() > 1) {
     nsim <- 10
     set.seed(2)
     system.time(
